@@ -294,12 +294,6 @@ static void cross_pressed(){
   chum_go_next();
 }
 
-static void back_pressed(){
-  chum_stop_sequencer();
-  chum_go_back();
-}
-
-
 static void init_song_table(){
   //Make one clickable image for each song
   if (group_of_songs!=NULL)
@@ -359,7 +353,7 @@ void init_singing_game(){
   ClutterActor* stage =main_getStage();
 
   //This is background. We want cover all underlying stuff. 
-  background =clutter_rectangle_new_with_color (get_green_color());
+  background =clutter_rectangle_new_with_color (get_sing_green_color());
   clutter_actor_set_size (background, WIDTH_OF_WINDOW,HEIGHT_OF_WINDOW);
   clutter_actor_set_position (background, 0, 0); 
   clutter_actor_hide (background);
@@ -447,7 +441,12 @@ void start_mobikid_recording(){
   printf("start_mobikid_recording starts\n");
   ClutterActor* stage =main_getStage();
 
-  clutter_actor_hide(group_of_songs);
+  clutter_actor_hide (to_main_menu);
+  clutter_actor_hide (group_of_songs);
+  clutter_actor_hide (mentor);
+  clutter_actor_hide (language_chooser);
+  clutter_actor_hide (duetto_solo);
+
 
   //Image of currently playing song
   //FIXME 'big_image' = background
@@ -471,7 +470,7 @@ void start_mobikid_recording(){
      }
   else
      clutter_texture_set_from_file (CLUTTER_TEXTURE (contour_star), filepath1, NULL);
-		 }
+     }
   else
      clutter_texture_set_from_file (CLUTTER_TEXTURE (contour_star), filepath1, NULL);
 
@@ -480,38 +479,24 @@ void start_mobikid_recording(){
   clutter_actor_show (big_image);
   clutter_container_add_actor (CLUTTER_CONTAINER (stage), big_image);
 
-  clutter_actor_show (display_freq);
-  clutter_text_set_text (CLUTTER_TEXT(display_freq),"");
-  clutter_actor_raise_top (display_freq);
-
   clutter_actor_show (contour_star);
   clutter_actor_raise_top (contour_star);
 
-  clutter_actor_hide (mentor);
-
-//  clutter_actor_show (microphone);
-//  clutter_actor_raise_top (microphone);
+  clutter_actor_show (display_freq);
+  clutter_text_set_text (CLUTTER_TEXT(display_freq),"");
+  clutter_actor_raise_top (display_freq);
 
   clutter_actor_show (cross);
   clutter_actor_raise_top (cross);
   clutter_actor_set_reactive(cross, TRUE);
 
-  clutter_actor_hide (language_chooser);
-//  clutter_actor_raise_top (back);
-//  clutter_actor_show (back);
-//  clutter_actor_set_reactive(back, TRUE);
+  clutter_actor_show(star_trace);
+  clutter_actor_raise_top (star_trace);
 
 
+  //recording_track is from chum.c. on_pitch_detected uses only globals
+  g_signal_connect(recording_track, "pitch-detected", G_CALLBACK(on_pitch_detected), NULL);
 
-	//recording_track is from chum.c. on_pitch_detected uses only globals
-	g_signal_connect(recording_track, "pitch-detected", G_CALLBACK(on_pitch_detected), NULL);
-
-
-	clutter_actor_show(star_trace);
-	clutter_actor_raise_top (star_trace);
-
-  clutter_actor_lower_bottom(to_main_menu);
-  clutter_actor_hide (to_main_menu);
 
   printf("will start sequencer\n");
   chum_play_sequencer();
@@ -519,7 +504,7 @@ void start_mobikid_recording(){
 
 gboolean singing_game_is_initialized(){
   //printf("singing_game_is_initialized called. will return %d\n",singinggame_back_to_mainmenu!=NULL);
-  return (language_chooser!=NULL); //last actor
+  return (cross!=NULL); //last actor
 }
 
 /*
