@@ -9,19 +9,13 @@ License:LGPL 2.1
  *
  */
 
-//This is because of getline
-#define _GNU_SOURCE
-
-#define VERSION "0.10.22"
-#define PACKAGE "unknown"
-#define GST_PACKAGE_NAME "unknown"
-#define GST_PACKAGE_ORIGIN "unknown" 
-
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
 #include <gst/controller/gstcontroller.h>
+#include <sys/time.h>
 
+#include "../plugin-common.h"
 #include "gstjammosampler.h"
 
 /*e.g. Nokia Internet Tablet n810 doesn't have this (maemo diablo)*/
@@ -39,9 +33,6 @@ License:LGPL 2.1
 
 #define M_PI_M2 ( M_PI + M_PI )
 
-GST_DEBUG_CATEGORY_STATIC (jammo_sampler_debug);
-#define GST_CAT_DEFAULT jammo_sampler_debug
-
 static const GstElementDetails gst_jammo_sampler_details =
 GST_ELEMENT_DETAILS ("JamMo Virtual Instrument",
     "Source/Audio",
@@ -70,6 +61,7 @@ enum
   PROP_TIMESTAMP_OFFSET,
   PROP_CAN_ACTIVATE_PUSH,
   PROP_CAN_ACTIVATE_PULL,
+  PROP_RECORDING,
   PROP_LAST
 };
 
@@ -151,8 +143,7 @@ return -1; //error case
 }
 
 
-static void
-gst_jammo_sampler_class_init (GstJammoSamplerClass * klass)
+static void gst_jammo_sampler_class_init (GstJammoSamplerClass * klass)
 {
   GObjectClass *gobject_class;
   GstBaseSrcClass *gstbasesrc_class;
@@ -215,7 +206,10 @@ gst_jammo_sampler_class_init (GstJammoSamplerClass * klass)
       g_param_spec_boolean ("can-activate-pull", "Can activate pull",
           "Can activate in pull mode", DEFAULT_CAN_ACTIVATE_PULL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
+  g_object_class_install_property (gobject_class, PROP_RECORDING,
+      g_param_spec_boolean ("realtime", "Realtime",
+          "Realtime", TRUE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   gstbasesrc_class->set_caps = GST_DEBUG_FUNCPTR (gst_jammo_sampler_setcaps);
   gstbasesrc_class->is_seekable =
       GST_DEBUG_FUNCPTR (gst_jammo_sampler_is_seekable);
@@ -299,8 +293,7 @@ fclose(filepointer);
 
 
 
-static void
-gst_jammo_sampler_init (GstJammoSampler * src, GstJammoSamplerClass * g_class)
+static void gst_jammo_sampler_init (GstJammoSampler * src, GstJammoSamplerClass * g_class)
 {
   gint i;
   GstPad *pad = GST_BASE_SRC_PAD (src);
@@ -344,8 +337,7 @@ gst_jammo_sampler_init (GstJammoSampler * src, GstJammoSamplerClass * g_class)
 
 }
 
-static void
-gst_jammo_sampler_src_fixate (GstPad * pad, GstCaps * caps)
+static void gst_jammo_sampler_src_fixate (GstPad * pad, GstCaps * caps)
 {
   GstJammoSampler *src = GST_JAMMO_SAMPLER (GST_PAD_PARENT (pad));
   const gchar *name;
@@ -364,10 +356,9 @@ gst_jammo_sampler_src_fixate (GstPad * pad, GstCaps * caps)
     gst_structure_fixate_field_nearest_int (structure, "width", 64);
 }
 
-static ProcessFunc process_funcs[];
+static JammoSamplerProcessFunc process_funcs[];
 
-static gboolean
-gst_jammo_sampler_setcaps (GstBaseSrc * basesrc, GstCaps * caps)
+static gboolean gst_jammo_sampler_setcaps (GstBaseSrc * basesrc, GstCaps * caps)
 {
   GstJammoSampler *src = GST_JAMMO_SAMPLER (basesrc);
   const GstStructure *structure;
@@ -424,8 +415,7 @@ gst_jammo_sampler_setcaps (GstBaseSrc * basesrc, GstCaps * caps)
   return ret;
 }
 
-static gboolean
-gst_jammo_sampler_query (GstBaseSrc * basesrc, GstQuery * query)
+static gboolean gst_jammo_sampler_query (GstBaseSrc * basesrc, GstQuery * query)
 {
   GstJammoSampler *src = GST_JAMMO_SAMPLER (basesrc);
   gboolean res = FALSE;
@@ -490,50 +480,8 @@ error:
 }
 
 
-gfloat pitch(gint needed_pitch){
-	gfloat pitch=1.0;
-		if (needed_pitch<=0)
-			while (needed_pitch<0) {
-				pitch*=0.9439;
-				needed_pitch++;
-			}
 
-		else
-			while (needed_pitch>0) {
-				pitch*=1.0595;
-				needed_pitch--;
-			}
-		
-	return pitch;
-}
-
-gfloat interpolation(gfloat interpolation_point, gfloat value1, gfloat value2) {
-	/* linear interpolation */
-	return ((1-interpolation_point)*value1 + interpolation_point*value2);
-}
-
-gfloat xfader(gint16 *sampletable, gfloat *xfadecounter, gfloat *xfadeindex, gint xfade, gfloat value, gfloat scale, gint needed_pitch, gint samplerate) {
-
-	gfloat xfademix;
-
-	/* linear crossfade */
-	/* fade in signal */
-	xfademix= (0.0+1.0* (*xfadecounter)/(xfade*1.0));
-	value*= xfademix;
-	/* fade out signal */
-	xfademix= (1.0-1.0* (*xfadecounter)/(xfade*1.0));
-	value+= (xfademix)*( interpolation( *xfadeindex-(gint)*xfadeindex, sampletable[(gint)*xfadeindex], sampletable[((gint)*xfadeindex+1)] ) /(G_MAXINT16*1.0) * scale);
-	/* Compensation of volume loss during crossfade */
-	/*values[counter]*=1.05-fabs(0.05-0.1* (*xfadecounter)/(*xfade*1.0));*/
-	*xfadecounter+=1.0;
-	/* 44100 is samplerate of sample files. value could be read from wav header and stored to i.e slicedWav->samplerate. then we would be able to used samples with different sample rates */
-	/* samplerate variable has the output samplerate of jammosampler */
-	*xfadeindex += pitch(needed_pitch)*44100/(samplerate+0.0);
-
-	return value;
-}
-
-gfloat looping_sampler(GstJammoSampler * src, gint j) {
+static gfloat looping_sampler(GstJammoSampler * src, gint j) {
 
 	gfloat *sampleindex, *xfadeindex, *xfadecounter;
 	gint *xfade;
@@ -624,7 +572,7 @@ gfloat looping_sampler(GstJammoSampler * src, gint j) {
 	return value;
 }
 
-gfloat nonlooping_sampler(GstJammoSampler * src, gint j) {
+static gfloat nonlooping_sampler(GstJammoSampler * src, gint j) {
 
 	gfloat *sampleindex;
 	gint16 *sampletable;
@@ -691,7 +639,7 @@ gfloat nonlooping_sampler(GstJammoSampler * src, gint j) {
 	return value;
 }
 
-gfloat oneshot_sampler(GstJammoSampler * src, gint j) {
+static gfloat oneshot_sampler(GstJammoSampler * src, gint j) {
 
 	gfloat *sampleindex;
 	gint16 *sampletable;
@@ -735,8 +683,52 @@ gfloat oneshot_sampler(GstJammoSampler * src, gint j) {
 	return value;
 }
 
+
+static void do_playback_process(GstJammoSampler * src){
+	GstClockTime now = gst_clock_get_time(gst_system_clock_obtain()); //time just now		
+	GstClockTime current = GST_CLOCK_DIFF(src->playback_starting_time,now);
+
+	JammoMidiEvent* event = g_list_nth(src->eventlist,src->playback_index_for_event)->data;
+	GstClockTime timestamp = event->timestamp;
+	/*
+	timestamp is for microseconds = µs
+	e.g.:
+	6.081 683 703 s
+	6 081.683 703 ms
+	6 081 683.703 ns
+	6 081 683 703 µs
+	*/
+	if (current > timestamp){
+		printf("DEBUG: gstjammosampler: delayed: %lu ms\n",(current - timestamp)/1000/1000); 
+		src->notes_playing++;
+
+		int octave=event->octave;
+		int note=event->note;
+
+		if (src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->filename!=NULL) {
+			src->note_table[octave*NUMBER_OF_NOTES+note]=TRUE;
+			src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->sampleindex=0;
+			src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->xfadecounter=-1.0;
+			src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->releasecounter=-1.0;
+			src->wav_based_instrument[octave*NUMBER_OF_NOTES+note]->released=FALSE;
+		}
+
+
+		src->playback_index_for_event++;
+		if (src->playback_index_for_event==g_list_length(src->eventlist))
+		{
+		src->eos_reached=TRUE; //FIXME: This stops it too early, that last note doesn't hear.
+		src->playback_not_started=TRUE;
+		}
+
+	}
+}
+
 #define DEFINE_PROCESS(type,scale) \
 static void gst_jammo_sampler_process_##type (GstJammoSampler * src, g##type * samples) {\
+	if (!(src->recording)) /*we are in playback mode*/ \
+		do_playback_process(src); \
+\
 	/* if volume is divided with number of playing tones instrument */ \
 	/* gets quieter when more simultaneous notes are played */ \
 \
@@ -813,16 +805,15 @@ DEFINE_PROCESS (float, 1.0);
 DEFINE_PROCESS (double, 1.0);
 
 
-static ProcessFunc process_funcs[] = {
-  (ProcessFunc) gst_jammo_sampler_process_int16,
-  (ProcessFunc) gst_jammo_sampler_process_int32,
-  (ProcessFunc) gst_jammo_sampler_process_float,
-  (ProcessFunc) gst_jammo_sampler_process_double
+static JammoSamplerProcessFunc process_funcs[] = {
+  (JammoSamplerProcessFunc) gst_jammo_sampler_process_int16,
+  (JammoSamplerProcessFunc) gst_jammo_sampler_process_int32,
+  (JammoSamplerProcessFunc) gst_jammo_sampler_process_float,
+  (JammoSamplerProcessFunc) gst_jammo_sampler_process_double
 };
 
 
-static void
-gst_jammo_sampler_get_times (GstBaseSrc * basesrc, GstBuffer * buffer,
+static void gst_jammo_sampler_get_times (GstBaseSrc * basesrc, GstBuffer * buffer,
     GstClockTime * start, GstClockTime * end)
 {
   /* for live sources, sync on the timestamp of the buffer */
@@ -844,12 +835,13 @@ gst_jammo_sampler_get_times (GstBaseSrc * basesrc, GstBuffer * buffer,
   }
 }
 
-static gboolean
-gst_jammo_sampler_start (GstBaseSrc * basesrc)
+static gboolean gst_jammo_sampler_start (GstBaseSrc * basesrc)
 {
   GstJammoSampler *src = GST_JAMMO_SAMPLER (basesrc);
   int i;
 
+  printf("DEBUG:gstjammosampler: jammo_sampler_starts\n");
+
   src->next_sample = 0;
   src->next_byte = 0;
   src->next_time = 0;
@@ -860,19 +852,43 @@ gst_jammo_sampler_start (GstBaseSrc * basesrc)
        src->accumulators[i] = 0;
 
 
+  //realtime recording
+  if (src->recording) {
+    printf("DEBUG:gstjammosampler: recording mode\n");
+    src->eventlist=NULL;
+    src->creation_time = gst_clock_get_time(gst_system_clock_obtain()); //time just now
+    src->playback_not_started=TRUE;
+  }
+  //playback
+  else  {
+    printf("DEBUG:gstjammosampler: not recording, playback\n");
+
+    src->playback_starting_time = gst_clock_get_time(gst_system_clock_obtain());
+    src->playback_not_started=FALSE;
+    src->playback_index_for_event=0;
+
+    //Test:
+   GList* list;
+   JammoMidiEvent* n;
+
+   for (list = src->eventlist; list; list = list->next) {
+      n = (list->data);
+      printf("TEST:gstjammosampler:In stack: %lu µs: note:%d octave: %d toggled\n", n->timestamp,n->note,n->octave); 
+   }
+  }
+
+
   return TRUE;
 }
 
-static gboolean
-gst_jammo_sampler_stop (GstBaseSrc * basesrc)
+static gboolean gst_jammo_sampler_stop (GstBaseSrc * basesrc)
 {
   return TRUE;
 }
 
 /* seek to time, will be called when we operate in push mode. In pull mode we
  * get the requiested byte offset. */
-static gboolean
-gst_jammo_sampler_do_seek (GstBaseSrc * basesrc, GstSegment * segment)
+static gboolean gst_jammo_sampler_do_seek (GstBaseSrc * basesrc, GstSegment * segment)
 {
   GstJammoSampler *src = GST_JAMMO_SAMPLER (basesrc);
   GstClockTime time;
@@ -902,15 +918,13 @@ gst_jammo_sampler_do_seek (GstBaseSrc * basesrc, GstSegment * segment)
   return TRUE;
 }
 
-static gboolean
-gst_jammo_sampler_is_seekable (GstBaseSrc * basesrc)
+static gboolean gst_jammo_sampler_is_seekable (GstBaseSrc * basesrc)
 {
   /* we're seekable... */
   return TRUE;
 }
 
-static gboolean
-gst_jammo_sampler_check_get_range (GstBaseSrc * basesrc)
+static gboolean gst_jammo_sampler_check_get_range (GstBaseSrc * basesrc)
 {
   GstJammoSampler *src;
 
@@ -920,8 +934,7 @@ gst_jammo_sampler_check_get_range (GstBaseSrc * basesrc)
   return src->can_activate_pull;
 }
 
-static GstFlowReturn
-gst_jammo_sampler_create (GstBaseSrc * basesrc, guint64 offset,
+static GstFlowReturn gst_jammo_sampler_create (GstBaseSrc * basesrc, guint64 offset,
     guint length, GstBuffer ** buffer)
 {
   GstFlowReturn res;
@@ -1083,6 +1096,36 @@ static void get_token(gchar **value){
 ////////////////////////////////////////////
 
 
+/*
+ Returns time difference between current time
+ and creation_time  
+*/
+struct timeval calc_timestamp (struct timeval creation_time) {
+  struct timeval result;
+
+  struct timeval now;
+  gettimeofday(&now, NULL);
+
+  if (now.tv_usec < creation_time.tv_usec) {
+    int nsec = (creation_time.tv_usec - now.tv_usec) / 1000000 + 1;
+    creation_time.tv_usec -= 1000000 * nsec;
+    creation_time.tv_sec += nsec;
+  }
+  if (now.tv_usec - creation_time.tv_usec > 1000000) {
+    int nsec = (now.tv_usec - creation_time.tv_usec) / 1000000;
+    creation_time.tv_usec += 1000000 * nsec;
+    creation_time.tv_sec -= nsec;
+  }
+
+  /* Compute the time.
+     tv_usec is certainly positive. */
+  result.tv_sec = now.tv_sec - creation_time.tv_sec;
+  result.tv_usec = now.tv_usec - creation_time.tv_usec;
+
+  return result;
+}
+
+
 static void
 gst_jammo_sampler_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -1253,6 +1296,22 @@ gst_jammo_sampler_set_property (GObject * object, guint prop_id,
 	//We must keep track how many notes are playing (we do not want count them separatedly)
 	if (new_value && !old_value) /*one FALSE is now TRUE*/
 	    src->notes_playing++;
+	
+	//Add this event to list.
+	JammoMidiEvent* event;
+
+	event = malloc(sizeof(JammoMidiEvent));
+	event->note=note;
+	event->octave=octave;
+
+	GstClockTime now = gst_clock_get_time(gst_system_clock_obtain()); //time just now	
+
+	event->timestamp=GST_CLOCK_DIFF(src->creation_time,now);
+	printf("DEBUG: gstjammosampler: added event %lu ms: note:%d octave:%d toggled\n",event->timestamp/1000/1000,event->note,event->octave); 
+	
+	src->eventlist = g_list_append(src->eventlist,event);
+
+	
 	break;
 
     case PROP_VOLUME:
@@ -1270,14 +1329,16 @@ gst_jammo_sampler_set_property (GObject * object, guint prop_id,
     case PROP_CAN_ACTIVATE_PULL:
       src->can_activate_pull = g_value_get_boolean (value);
       break;
+    case PROP_RECORDING:
+      src->recording = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
-static void
-gst_jammo_sampler_get_property (GObject * object, guint prop_id,
+static void gst_jammo_sampler_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
   GstJammoSampler *src = GST_JAMMO_SAMPLER (object);
@@ -1328,28 +1389,11 @@ gst_jammo_sampler_get_property (GObject * object, guint prop_id,
     case PROP_CAN_ACTIVATE_PULL:
       g_value_set_boolean (value, src->can_activate_pull);
       break;
+    case PROP_RECORDING:
+      g_value_set_boolean (value, src->recording);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
-
-static gboolean
-plugin_init (GstPlugin * plugin)
-{
-
-	
-  /* initialize gst controller library */
-  gst_controller_init (NULL, NULL);
-
-  GST_DEBUG_CATEGORY_INIT (jammo_sampler_debug, "jammosampler", 0,
-      "Jammo Virtual Instrument");
-  return gst_element_register (plugin, "jammosampler",
-      GST_RANK_NONE, GST_TYPE_JAMMO_SAMPLER);
-}
-
-GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
-    GST_VERSION_MINOR,
-    "jammosampler",
-    "Creates virtual instrument",
-    plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
