@@ -25,7 +25,6 @@ enum {
 struct _JammoInstrumentTrackPrivate {
 	GstElement* element;
 	int instrument_type;
-	struct timeval creation_time;
 };
 
 JammoInstrumentTrack* jammo_instrument_track_new(JammoInstrumentType instrument_type) {
@@ -118,8 +117,7 @@ static GObject* jammo_instrument_track_constructor(GType type, guint n_propertie
 
 	g_object_set(G_OBJECT(instrument_track->priv->element), "init-instrument", buffer, NULL);
 
-	gettimeofday(&instrument_track->priv->creation_time, NULL);
-
+	g_object_set(G_OBJECT(instrument_track->priv->element), "realtime", TRUE, NULL);
 	return object;
 }
 
@@ -149,48 +147,20 @@ static void jammo_instrument_track_class_init(JammoInstrumentTrackClass* instrum
 	g_type_class_add_private(gobject_class, sizeof(JammoInstrumentTrackPrivate));
 }
 
-
-/*
- Returns time difference between current time
- and instrument_track->priv->creation_time  
-*/
-struct timeval calc_timestamp (JammoInstrumentTrack* instrument_track) {
-  struct timeval result;
-
-  struct timeval now;
-  gettimeofday(&now, NULL);
-
-  if (now.tv_usec < instrument_track->priv->creation_time.tv_usec) {
-    int nsec = (instrument_track->priv->creation_time.tv_usec - now.tv_usec) / 1000000 + 1;
-    instrument_track->priv->creation_time.tv_usec -= 1000000 * nsec;
-    instrument_track->priv->creation_time.tv_sec += nsec;
-  }
-  if (now.tv_usec - instrument_track->priv->creation_time.tv_usec > 1000000) {
-    int nsec = (now.tv_usec - instrument_track->priv->creation_time.tv_usec) / 1000000;
-    instrument_track->priv->creation_time.tv_usec += 1000000 * nsec;
-    instrument_track->priv->creation_time.tv_sec -= nsec;
-  }
-
-  /* Compute the time.
-     tv_usec is certainly positive. */
-  result.tv_sec = now.tv_sec - instrument_track->priv->creation_time.tv_sec;
-  result.tv_usec = now.tv_usec - instrument_track->priv->creation_time.tv_usec;
-
-  return result;
-}
-
-
-void jammo_instrument_track_set_note(JammoInstrumentTrack* instrument_track,char note,int octave){
+void jammo_instrument_track_set_and_play_note_realtime(JammoInstrumentTrack* instrument_track, char note, int octave) {
 	gchar *buffer0;
 	buffer0 = g_strdup_printf("%c%d",note,octave);
 	g_object_set(G_OBJECT(instrument_track->priv->element), "note-toggle", buffer0, NULL);
-
-	struct timeval timestamp;
-	timestamp=calc_timestamp(instrument_track);
-	printf("INSTRUMENT_TRACK: %ld.%ld: note %c%d toggled\n", timestamp.tv_sec,timestamp.tv_usec,note,octave); 
+	g_free(buffer0);
 }
 
 static void jammo_instrument_track_init(JammoInstrumentTrack* instrument_track) {
 	instrument_track->priv = G_TYPE_INSTANCE_GET_PRIVATE(instrument_track, JAMMO_TYPE_INSTRUMENT_TRACK, JammoInstrumentTrackPrivate);
 }
 
+
+void jammo_instrument_track_set_realtime(JammoInstrumentTrack* instrument_track, gboolean state){
+	g_object_set(G_OBJECT(instrument_track->priv->element), "realtime", state, NULL);
+}
+
+
